% Comparing the filter described in 
% W. C. Moss, S. Haase, J. M. Lyle, D. A. Agard, and J. W. Sedat. A novel 3D
% wavelet-based filter for visualizing features in noisy biological data. Journal
% of Microscopy, 219(2):4349, 2005.
% with a bunch of other filters

% (c)2005 by Cris Luengo

fsz = 6;

% Signal
a = newim;
a = drawline(a,[40,40],[70,200],1);
a = drawline(a,[40,100],[200,160],1);
a = drawline(a,[170,50],[170,210],1);
signal = a+dilation(a,3);
signal = signal+8*dilation(a,5);

% Interesting regions
m_bgr = ~bdilation(signal>0,fsz*3,-1,1);
%m_sig = bdilation(signal>0,ceil(fsz/2),1,0);
m_sig = signal>0;
k_sig = double(signal(m_sig)/10)';

for snr = [8,2,0.5];

disp([' * SNR = ',num2str(snr)]);

% Noisy input image
a = noise(signal,'gaussian',10/snr)/10;
disp(['Input image                    ', num2str(1/std(a(m_bgr)),'%.2f'),' (',num2str(10*mean((a(m_sig)-k_sig).^2),'%.3f'),')'])
%dipshow(a,[0,255]);
a = stretch(a);

% Moss' filter
b = mossfilter(a,fsz);
b = (double(b(m_sig))'\k_sig) * b;
%dipshow(b,[0,1]);
disp(['Moss'' filter                   ',num2str(1/std(b(m_bgr)),'%.2f'),' (',num2str(10*mean((b(m_sig)-k_sig).^2),'%.3f'),')'])

% LoG
c = -laplace(a,fsz*0.6);
c = clip(c,0,inf);
c = (double(c(m_sig))'\k_sig) * c;
%dipshow(c,[0,1]);
disp(['Laplace of Gaussian            ', num2str(1/std(c(m_bgr)),'%.2f'),' (',num2str(10*mean((c(m_sig)-k_sig).^2),'%.3f'),')'])

% DoU
d = unif(a,fsz)-unif(a,3*fsz);
d = clip(d,0,inf);
d = (double(d(m_sig))'\k_sig) * d;
%dipshow(d,[0,1]);
disp(['Difference of uniform filters  ', num2str(1/std(d(m_bgr)),'%.2f'),' (',num2str(10*mean((d(m_sig)-k_sig).^2),'%.3f'),')'])

% Median filter
e = medif(a,fsz-1);
e = clip(e-median(a),0,inf);
e = (double(e(m_sig))'\k_sig) * e;
%dipshow(e,[0,1]);
disp(['Median filter                  ', num2str(1/std(e(m_bgr)),'%.2f'),' (',num2str(10*mean((e(m_sig)-k_sig).^2),'%.3f'),')'])

% Kuwahara
f = kuwahara(a,fsz-1);
f = clip(f-median(a),0,inf);
f = (double(f(m_sig))'\k_sig) * f;
%dipshow(f,[0,1]);
disp(['Kuwahara-Nagao operator        ', num2str(1/std(f(m_bgr)),'%.2f'),' (',num2str(10*mean((f(m_sig)-k_sig).^2),'%.3f'),')'])

% Alternating sequential filter
g = a;
for ii=2:fsz-3
   g = opening(g,ii);
   g = closing(g,ii);
end
g = clip(g-median(a),0,inf);
g = (double(g(m_sig))'\k_sig) * g;
%dipshow(g,[0,1]);
disp(['Sequential open-close          ', num2str(1/std(g(m_bgr)),'%.2f'),' (',num2str(10*mean((g(m_sig)-k_sig).^2),'%.3f'),')'])

end
