% MOSSFILTER   A wavelet-based line-detection filter
%    As described in:
%    W. C. Moss, S. Haase, J. M. Lyle, D. A. Agard, and J. W. Sedat.
%    A novel 3D wavelet-based filter for visualizing features in noisy
%    biological data. Journal of Microscopy, 219(2):4349, 2005.

% (c)2005 by Cris Luengo

function out = mossfilter(in,a)

% R(0)
out1 = basicfilter(in,a);

% R(45)
out2 = rotation(in,pi/4);
out2 = basicfilter(out2,a);
out2 = rotation(out2,-pi/4);
% After rotating an image 45 degrees and back it is way bigger. Crop.
sz = size(in);
osz = size(out2);
osz = (osz-sz)/2;
out2 = out2(osz(1):osz(1)+sz(1)-1,osz(2):osz(2)+sz(2)-1);
% The filter rotated 45 degrees has some heavy edge effects. We'll set the edges to 0.
out2(:,[0:a-1,end-a+1:end]) = 0;
out2([0:a-1,end-a+1:end],:) = 0;

% Average R(0) and R(45)
out = ( out1 + out2 ) / 2;

% R(phi)
function out = basicfilter(in,a)

% W(x,y)
out1 = unif(in,[a,1],'rectangular') - unif(in,[3*a,1],'rectangular');
out1 = clip(out1,0,inf);
out1 = unif(out1,[1,a],'rectangular') - unif(out1,[1,3*a],'rectangular');
out1 = clip(out1,0,inf);

% W(y,x)
out2 = unif(in,[1,a],'rectangular') - unif(in,[1,3*a],'rectangular');
out2 = clip(out2,0,inf);
out2 = unif(out2,[a,1],'rectangular') - unif(out2,[3*a,1],'rectangular');
out2 = clip(out2,0,inf);

% Average W(x,y) and W(y,x)
out = ( out1 + out2 ) / 2;
